#lang scheme

(require (planet schematics/schemeunit:3:3)
         "bruces-mipmap.scm")


;; Texture related tests - start

(define test-texture
  (make-texture
   (string->color% "Brown") (string->color% "Yellow")
   320 480
   1 17
   100
   'circle
   (string->color% "Orange")
   (string->color% "Green")))

(define test-textures (next-textures test-texture 5))

(define min-dot-size 1)

;; width and height must always be even.
;; odd dot-size is accounted for here
;; even dot count is accounted for here
(test-begin
 (let ((texture (car test-textures)))
   (check equal? (texture-bg-color texture) (string->color% "Brown"))
   (check equal? (texture-fg-color texture) (string->color% "Yellow"))
   (check equal? (texture-width texture) 320)
   (check equal? (texture-height texture) 480)
   (check equal? (texture-min-dot-size texture) min-dot-size)
   (check equal? (texture-max-dot-size texture) 17)
   (check equal? (texture-dot-count texture) 100)
   (check equal? (texture-dot-shape texture) 'circle)
   (check equal? (texture-debug-color texture) 
          (string->color% "Orange"))
   (check equal? (texture-dot-debug-color texture) 
          (string->color% "Green"))))

(test-begin
 (let ((texture (cadr test-textures)))
   (check equal? (texture-width texture) 160)
   (check equal? (texture-height texture) 240)
   (check equal? (texture-min-dot-size texture) min-dot-size)
   (check equal? (texture-max-dot-size texture) 9)
   (check equal? (texture-dot-count texture) 50)))

(test-begin
 (let ((texture (caddr test-textures)))
   (check equal? (texture-width texture) 80)
   (check equal? (texture-height texture) 120)
   (check equal? (texture-min-dot-size texture) min-dot-size)
   (check equal? (texture-max-dot-size texture) 5)
   (check equal? (texture-dot-count texture) 25)))

;; odd dot count is accounted for here
(test-begin
 (let ((texture (cadddr test-textures)))
   (check equal? (texture-width texture) 40) 
   (check equal? (texture-height texture) 60)
   (check equal? (texture-min-dot-size texture) min-dot-size)
   (check equal? (texture-max-dot-size texture) 3)
   (check equal? (texture-dot-count texture) 13)))

(define test-even-dot-size-texture
  (make-texture
   (string->color% "Brown") (string->color% "Yellow")
   320 480
   1 16
   100
   'circle
   (string->color% "Orange")
   (string->color% "Red")))

(define even-dot-size-textures 
  (next-textures test-even-dot-size-texture 1))

;; even dot-size is accounted for here
(test-begin
 (let ((texture (cadr even-dot-size-textures)))
   (check equal? (texture-max-dot-size texture) 8)))

;; Texture related tests - stop

;; Dot related tests - start

; Rectangle testing

(define root-square (make-dot 30 30 10 'square))

;; These rectangles all but up right against the end of
;; the four corners of the root-square, but do not overlap.
(define top-left-no (make-dot 20 20 10 'square))
(define bottom-left-no (make-dot 20 40 10 'square))
(define top-right-no (make-dot 40 20 10 'square))
(define bottom-right-no (make-dot 40 40 10 'square))

(check equal? (rectangles-overlap? root-square top-left-no) #f)
(check equal? (rectangles-overlap? top-left-no root-square) #f)
(check equal? (rectangles-overlap? root-square bottom-left-no) #f)
(check equal? (rectangles-overlap? bottom-left-no root-square) #f)
(check equal? (rectangles-overlap? root-square top-right-no) #f)
(check equal? (rectangles-overlap? top-right-no root-square) #f)
(check equal? (rectangles-overlap? root-square bottom-right-no) #f)
(check equal? (rectangles-overlap? bottom-right-no root-square) #f)

;; TODO Add tests for overlapping rectangles

;; TODO Add tests for overlapping and non-overlapping circles

;; Dot related tests - stop

(check equal? (distance (make-posn 1 2) (make-posn 2 2)) 1)
(check equal? (distance (make-posn 1 2) (make-posn 3 2)) 2)
(check equal? (distance (make-posn 1 2) (make-posn 4 2)) 3)